import rich_click as click
from pathlib import Path
from lid_search.lid_search import LidDB,lid_builder
from semantic_version_tools import Vers
from xml.dom.minidom import parse


def getFromXml(xml, label, idx=0):
    a = xml.getElementsByTagName(label)[idx]
    return a.firstChild.nodeValue

def get_lid(filename:Path)->str:
    tree = parse(filename.__str__())
    lid=getFromXml(tree, 'logical_identifier')
    return lid

@click.group()
def main():
    pass

@main.command
@click.argument("filename", type=click.Path(exists=True, file_okay=True, dir_okay=False, readable=True, path_type=Path))
@click.option(
    "-d",
    "--database",
    type=click.Path(
        exists=False, file_okay=True, dir_okay=False, readable=True, path_type=Path
    ),
    help="Path and name of the database file",
)
@click.option('-p','--preserve', is_flag=True, help="Preserve the database file",default=False)
@click.option('-c','--cache', is_flag=True, help="Use the cache file",default=False)
def search(filename:Path,database:Path,preserve:bool,cache:bool):
    """Get the current version of the lid"""
    lid=get_lid(filename)
    # print(lid)
    
    db=LidDB(database,cache=cache)
        
    dat=db.search(lid)    
    if dat:
        v=Vers(dat)
    print(f"Current version: {v.short()}, Next Version {v+1}")
    
    db.close(preserve)
    
@main.command
@click.argument("filename", type=click.Path(exists=True, file_okay=True, dir_okay=False, readable=True, path_type=Path))
def getlid(filename:Path):
    """Extract the LID from the XML file"""
    # lid=lid_builder(filename)
    # print(lid)
    lid2=get_lid(filename)
    print(lid2)


from pathlib import Path
import json
from rich import print
import sqlite3
from semantic_version_tools import Vers 
from shutil import rmtree
from importlib.metadata import version

__version__=version('lid_search')

class LidDB:
    
    def __init__(self,jFile:Path,temporary_folder:Path=Path('tmp'),cache:bool=False):
        if isinstance(jFile,str):
            jFile=Path(jFile)
        if isinstance(temporary_folder,str):
            temporary_folder=Path(temporary_folder)
        if not temporary_folder.exists():
            temporary_folder.mkdir()
        
        self.temporary_folder=temporary_folder
        self.db=sqlite3.connect(temporary_folder.joinpath('db.sqlite'))
        self.cursor=self.db.cursor()
        if not cache:
            jdata = json.loads(jFile.read_text())
            self.cursor.execute('CREATE TABLE data (id INTEGER PRIMARY KEY, lid TEXT, version Text)')
            for i in jdata['summary']:
                self.cursor.execute('INSERT INTO data (lid, version) VALUES (?,?)', (i['logicalIdentifier'], i['versionId']))
            self.db.commit()
            
    def search(self,lid:str):
        self.cursor.execute('SELECT * FROM data WHERE lid=?', (lid,))
        result=self.cursor.fetchall()
        if len(result)==0:
            ret=None
        else:
            # print(len(result))
            ret=result[0][2]
        return ret
    
    def close(self,preserve:bool=False):
        self.db.close()
        if not preserve:
            rmtree(self.temporary_folder,)

def lid_builder(filename:Path,collection:str=None,instrument:str='simbio-sys',mission:str='bc_mpo'):
    if collection is None:
        parts=filename.stem.split('_')
        if parts[2] =='sc':
            if parts[1]=='raw':
                collection='data_raw'
            else:
                collection='data_calibrated'
    
    return f"urn:esa:psa:{mission}_{instrument}:{collection}:{filename.stem}"